# Configuration file for Sphinx documentation builder.

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

import os
import sys
sys.path.insert(0, os.path.abspath('..'))


# -- Project information -----------------------------------------------------
project = 'kosu'
copyright = '2022, Agile Scientific'
author = 'Agile Scientific'


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.githubpages',
    'sphinx.ext.napoleon',
    'myst_parser',
    'sphinx.ext.coverage', 
]
 
# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'notebooks']


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# https://sphinx-themes.org/sample-sites/furo/
html_theme = 'furo'

html_theme_options = {
    "sidebar_hide_name": True,
    "footer_icons": [
        {
            "name": "Agile",
            "url": "https://code.agilescientific.com",
            "html": """
                <svg width="200" height="200" version="1.1" viewBox="0 0 187.5 187.5" xmlns="http://www.w3.org/2000/svg">
                <g transform="matrix(3.94 -1.28 1.28 3.94 -906.1 -3676)" style="color-rendering:auto;color:#000000;fill:#14ca29;image-rendering:auto;isolation:auto;mix-blend-mode:normal;shape-rendering:auto;solid-color:#000000;stroke-width:1px;text-decoration-color:#000000;text-decoration-line:none;text-decoration-style:solid;text-indent:0;text-transform:none;white-space:normal">
                <path d="m-46.98 936.1q3.515-1.622 6.705-3.244t5.083-2.379 3.028-0.757q1.893 0 3.244 1.298 1.406 1.244 1.406 3.136 0 1.081-0.7029 2.271-0.6489 1.136-1.406 1.46-6.975 2.758-15.36 4.001 1.514 1.406 3.731 3.731t2.325 2.487q0.8111 1.136 2.271 2.812 1.46 1.676 2.001 2.65 0.5948 0.9192 0.5948 2.271 0 1.73-1.298 3.028t-3.352 1.298q-2.055 0-4.65-3.19-2.541-3.19-6.597-11.46-4.109 7.462-5.515 9.841t-2.704 3.623q-1.298 1.19-2.974 1.19-2.001 0-3.352-1.352-1.298-1.406-1.298-2.974 0-1.46 0.5407-2.217 4.975-6.759 10.38-11.73-4.542-0.7029-8.111-1.568t-7.57-2.541q-0.6489-0.3244-1.298-1.46-0.5948-1.19-0.5948-2.163 0-1.893 1.352-3.136 1.406-1.298 3.19-1.298 1.298 0 3.244 0.8111 1.947 0.757 4.921 2.271 3.028 1.46 6.867 3.298-0.7029-3.407-1.19-7.786-0.4326-4.434-0.4326-6.056 0-2.001 1.244-3.407 1.298-1.46 3.298-1.46 1.947 0 3.19 1.46 1.298 1.406 1.298 3.785 0 0.6489-0.2163 2.595-0.1622 1.893-0.5407 4.65-0.3244 2.704-0.757 6.218z" style="fill:#14ca29"/>
                </g>
                </svg>
            """,
            "class": "",
        },
    ],
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

html_css_files = [
    'custom.css',
]

# Branding.
html_favicon = '_static/favicon.ico'
html_logo = '_static/kosu_logo.png'



import json
import os
import re

import requests


def hide_cells(notebook, tags=None):
    """
    Finds the tags in each cell and removes it.

    Returns dict without 'hide' tagged cells.
    """
    if tags is None:
        tags = ['hide']

    clean = []
    for cell in notebook['cells']:
        if not set(tags).intersection(cell['metadata'].get('tags', list())):
            clean.append(cell)

    notebook['cells'] = clean

    return notebook


def empty_cells(notebook):
    """
    Finds the tag 'empty' in each cell and removes its content

    Returns dict with empty cells
    """

    clean = []
    for cell in notebook['cells']:
        try:
            tags = cell['metadata']['tags']
            if True in map(lambda x: x.lower().startswith('empty'), tags):
                cell['source'] = []
                clean.append(cell)

        except KeyError:
            clean.append(cell)

    notebook['cells'] = clean

    return notebook


def style_cells(notebook, style):
    """
    Finds the tags 'exercise', 'advanced' or 'info' in each cell and applies HTML template.

    Returns dict with template cells.
    """
    styles = {
        'exercise': "<div style=\"background: #e0ffe0; border: solid 2px #d0f0d0; border-radius:3px; padding: 1em; color: darkgreen\">\n",
        'advanced': "<div style=\"background: #fff0e0; border: solid 2px #ffe7d0; border-radius:3px; padding: 1em; color: chocolate\">\n",
        'info': "<div style=\"background: #e0f0ff; border: solid 2px #d0e0f0; border-radius:3px; padding: 1em; color: navy\">\n",
    }

    clean = []
    wraphead = [styles[style], ]
    wraptail = ["\n</div>", ]

    for cell in notebook['cells']:
        try:
            tags = cell['metadata']['tags']
            if True in [t.lower().startswith(style[:3]) for t in tags]:
                src = cell['source']
                src = [re.sub(r"^#+? (.+)\n", r"<h3>\1</h3>\n", s) for s in src]
                cell['source'] = wraphead + src + wraptail
        except KeyError:
            pass
        clean.append(cell)
    notebook['cells'] = clean

    return notebook


def hide_code(notebook):
    """
    Finds the tags '#!--' and '#--! in each cell and removes
    the lines in between.

    Returns dict
    """

    for i, cell in enumerate(notebook['cells']):
        istart = 0
        istop = -1
        for idx, line in enumerate(cell['source']):
            if '#!--' in line:
                istart = idx
            if '#--!' in line:
                istop = idx

        notebook['cells'][i]['source'] = cell['source'][:istart] + cell['source'][istop+1:]

    return notebook


def hide_toolbar(notebook):
    """
    Finds the display toolbar tag and hides it
    """

    if 'celltoolbar' in notebook['metadata']:
        del(notebook['metadata']['celltoolbar'])
    return notebook


def process_notebook(infile,
                     outfile,
                     clear_input=False,  # Don't touch the input file.
                     clear_output=True,
                     exercise=True,
                     advanced=True,
                     info=True,
                     hidecode=True,
                     demo=False,  # If demo, remove exercises and enable demos.
                     data_url_stem=None,
                    ):
    """
    Loads an 'ipynb' file as a dict and performs cleaning tasks

    Writes cleaned version
    """
    if clear_input:
        _ = os.system("nbstripout {}".format(infile))

    with open(infile, encoding='utf-8') as f:
        notebook = json.loads(f.read())

    if demo:
        notebook = hide_cells(notebook, tags=['exercise', 'solution'])
    else:
        notebook = hide_cells(notebook, tags=['hide', 'demo'])

    if exercise:
        notebook = style_cells(notebook, 'exercise')
    if advanced:
        notebook = style_cells(notebook, 'advanced')
    if info:
        notebook = style_cells(notebook, 'info')

    if hidecode:
        notebook = hide_code(notebook)

    notebook = hide_toolbar(notebook)

    text = json.dumps(notebook)
    images = re.findall(r"\.\./images/([-_.a-zA-Z0-9]+)", text)
    # print(infile, images)

    if data_url_stem is None:
        data_url_stem = r"https://geocomp\.s3\.amazonaws\.com/data/"
    data_urls = re.findall(fr"({data_url_stem}[-_.a-zA-Z0-9]+)", text)
    # print(infile, data_urls)

    with open(outfile, 'w') as f:
        _ = f.write(text)

    if clear_output:
        _ = os.system("nbstripout {}".format(outfile))

    return images, data_urls

if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description="Clean a notebook")
    parser.add_argument('infile', help='Input notebook')
    parser.add_argument('outfile', help='Output notebook')
    parser.add_argument('--clear-input', action='store_true', help='Remove input cells')
    parser.add_argument('--clear-output', action='store_true', help='Remove output cells')

    process_notebook(**vars(parser.parse_args()))



"""
Author: Agile Scientific
Licence: Apache 2.0
"""
import pathlib
import shutil
import zipfile
import os
import warnings
from urllib.request import urlretrieve
import inspect
import sys
import glob

import requests
import click
import yaml
from jinja2 import Environment, FileSystemLoader

try:
    import boto3
    from botocore.exceptions import ClientError
    AWS_AVAILABLE = True
except:
    AWS_AVAILABLE = False

from .customize import process_notebook

env = Environment(loader=FileSystemLoader('templates'))


def removesuffix(s, suffix):
    """
    Remove the suffix from a string, if it exists.
    Avoid needing Python 3.9 just for this.
    """
    return s[:-len(suffix)] if s.endswith(suffix) else s


def get_script_dir(follow_symlinks=True):
    if getattr(sys, 'frozen', False):
        path = os.path.abspath(sys.executable)
    else:
        path = inspect.getabsfile(get_script_dir)
    if follow_symlinks:
        path = os.path.realpath(path)
    return pathlib.Path(os.path.dirname(path))


# Read the YAML control file, if present.
KOSU = {'path': get_script_dir()}
kosu = pathlib.Path(".kosu.yaml")
if kosu.is_file():
    with open(kosu, 'rt') as f:
        try:
            KOSU.update(yaml.safe_load(f))
        except yaml.YAMLError as e:
            print(e)

@click.group(context_settings=dict(help_option_names=['--help', '-h']))
def cli():
    pass


@cli.command()
@click.option('--yes', default=False, is_flag=True, help="Automatically confirm.")
def init(yes):
    """
    Initialize a course or group of courses.
    """
    if pathlib.Path('.kosu.yaml').exists():
        message = "There's already a .kosu.yml file in this directory. "
        message += "Please run this command in an empty directory."
        raise click.ClickException(message)

    message = "This will create a new course collection in the current directory. Are you sure?"
    if yes or click.confirm(message, default=False, abort=True):
        # Make folders for content.
        folders = ['notebooks', 'images', 'references', 'scripts', 'templates']
        for folder in folders:
            _ = pathlib.Path(folder).mkdir(exist_ok=True)

        # Make a sample course and other files.
        target = pathlib.Path('.').resolve()
        shutil.copyfile(KOSU['path'] / 'include' / 'example_course.yaml', target / 'example_course.yaml')
        shutil.copyfile(KOSU['path'] / 'include' / '.kosu.yaml', target / '.kosu.yaml')
        shutil.copyfile(KOSU['path'] / 'include' / 'environment.yaml', target / 'environment.yaml')
        shutil.copyfile(KOSU['path'] / 'include' / 'useful.pdf', target / 'references'/ 'useful.pdf')
        shutil.copyfile(KOSU['path'] / 'include' / 'README.md', target / 'templates' / 'README.md')
        shutil.copyfile(KOSU['path'] / 'include' / 'Interesting_notebook.ipynb', target / 'notebooks' / 'Interesting_notebook.ipynb')
        shutil.copyfile(KOSU['path'] / 'include' / 'Intro_to_Matplotlib.ipynb', target / 'notebooks' / 'Intro_to_Matplotlib.ipynb')
        shutil.copyfile(KOSU['path'] / 'include' / 'Intro_to_NumPy.ipynb', target / 'notebooks' / 'Intro_to_NumPy.ipynb')
        shutil.copyfile(KOSU['path'] / 'include' / 'Intro_to_Python.ipynb', target / 'notebooks' / 'Intro_to_Python.ipynb')
        shutil.copyfile(KOSU['path'] / 'include' / 'example.png', target / 'images' / 'example.png')
        shutil.copyfile(KOSU['path'] / 'include' / 'example.py', target / 'scripts' / 'example.py')
        click.secho("Created example course and config files.\n", fg="green", bold=True)
        click.secho("See .kosu.yaml for configuration options.", fg="cyan")

    return


@cli.command()
@click.argument('course', type=str, required=False)
@click.option('--all', is_flag=True, help="Cleans all courses listed in control file.")
def clean(course, all):
    """
    Clean COURSE builds from local storage.
    """
    courses = get_courses(course, all)

    for i, course in enumerate(courses):
        click.secho(f"Cleaning {course} ({i+1} of {len(courses)}).", fg="cyan")
        try:
            shutil.rmtree(pathlib.Path('build').joinpath(course))
        except FileNotFoundError:
            pass
        try:
            pathlib.Path(f'{course}.zip').unlink()
        except FileNotFoundError:
            pass
        try:
            if not any(pathlib.Path('build').iterdir()):
                click.secho(f"Removing build directory.", fg="red")
                shutil.rmtree(pathlib.Path('build'))
                break
        except:
            pass
    click.secho(f"Finished.\n", fg="green")

    return


@cli.command()
@click.argument('course', type=str, required=False)
@click.option('--all', is_flag=True, help="Publishes all courses listed in control file.")
def publish(course, all):
    """
    Publish COURSE to AWS.
    """
    courses = get_courses(course, all)

    for i, course in enumerate(courses):
        click.secho(f"Publishing {course} ({i+1}/{len(courses)}). Ctrl-C to abort.", fg="cyan")
        _ = build_course(course, clean=True, zip=True, upload=True, clobber=True)
    click.secho(f"Finished.\n", fg="green")

    return


@cli.command()
@click.argument('course', type=str, required=False)
@click.option('--all', is_flag=True, help="Tests all courses listed in control file.")
@click.option('--environment', is_flag=True, help="Build a global environment file for testing.")
def test(course, all, environment):
    """
    Test that COURSE builds without error.
    """
    courses = get_courses(course, all)

    envs = []
    for i, course in enumerate(courses):
        clean = 1 - environment  # Clean if we're not doing env.
        click.secho(f"Testing {course} ({i+1}/{len(courses)}). Ctrl-C to abort.", fg="cyan")
        env = build_course(course, clean=clean, zip=False, upload=False, clobber=True)
        envs.append(env)
    click.secho(f"Finished.\n", fg="green")

    if environment:
        # Build the combined environment.
        channels, conda, pip = set(), set(), set()
        for env in envs:
            channels.update(env['channels'])
            conda.update(env['dependencies'][:-1])  # All except pip.
            pip.update(env['dependencies'][-1]['pip'])
        env = {
            'name': 'kosu-all',
            'channels': list(channels),
            'dependencies': list(conda) + [{'pip': list(pip)}],
        }
        with open('environment-all.yml', 'w') as f:
            f.write(yaml.dump(env, default_flow_style=False, sort_keys=False))
        click.secho(f"Global environment file written.\n", fg="green")

    return


@cli.command()
@click.argument('course', type=str, required=False)
@click.option('--clean/--no-clean', default=True, help="Delete the build dir? Default: clean.")
@click.option('--zip/--no-zip', default=True, help="Make the zip file? Default: zip.")
@click.option('--upload/--no-upload', default=False, help="Upload the ZIP to S3? Default: no-upload.")
@click.option('--clobber/--no-clobber', default=False, help="Clobber existing files? Default: no-clobber.")
@click.option('--all', is_flag=True, help="Tests all courses listed in control file.")
def build(course, clean, zip, upload, clobber, all):
    """
    Build COURSE with various options.
    """
    courses = get_courses(course, all)

    for i, course in enumerate(courses):
        click.secho(f"Building {course} ({i+1}/{len(courses)}). Ctrl-C to abort.", fg="cyan")
        _ = build_course(course, clean, zip, upload, clobber)
    click.secho(f"Finished.\n", fg="green")

    return

# =============================================================================
def get_courses(course, all):
    """
    Returns the list of courses to process.
    """
    if (not all) and (course is None):
        message = "Missing argument 'COURSE', or use '--all'."
        raise click.UsageError(message)
    elif all and (course is not None):
        message = "'--all' cannot be used with 'COURSE'; use one or the other."
        raise click.BadOptionUsage('--all', message)
    elif all and (course is None):
        excl = ['environment.yaml']
        all_courses = [removesuffix(c, '.yaml') for c in glob.glob("*.yaml") if c not in excl]
        courses = KOSU.get('all', all_courses)
    else:
        courses = [removesuffix(course, '.yaml')]

    return courses


def build_course(course, clean, zip, upload, clobber):
    """
    Compiles the required files into a course repo, which
    will be zipped by default.

    Args:
        course (str): The course to build.
        clean (bool): Whether to remove the build files.
        zip (bool): Whether to create the zip file for the course repo (or to save it if uploading).
        upload (bool): Whether to attempt to upload the ZIP to AWS.
        clobber (bool): Whether to overwrite existing ZIP file and build directory.

    Returns:
        dict. Environment dictionary.
    """
    # Read the YAML control file.
    with open(f"{removesuffix(course, '.yaml')}.yaml", 'rt') as f:
        try:
            config = yaml.safe_load(f)
        except yaml.YAMLError as e:
            print(e)
    config['course'] = course

    # Make a path to store everything.
    build_path = 'build'
    path = pathlib.Path(build_path).joinpath(course)
    if path.exists():
        message = "The target directory exists and will be overwritten. Are you sure?"
        if clobber or click.confirm(message, default=True, abort=True):
            shutil.rmtree(path)
    if pathlib.Path(f"{course}.zip").exists():
        message = "The ZIP file exists and will be overwritten. Are you sure?"
        if clobber or click.confirm(message, default=True, abort=True):
            pathlib.Path(f"{course}.zip").unlink()

    _ = path.mkdir(parents=True, exist_ok=True)

    # Build the notebooks; also deals with images.
    *paths, _, data_urls_to_check = build_notebooks(path, config)

    # Check the data files exist.
    click.secho('Checking and downloading data ', fg="cyan", nl=False)
    for url in data_urls_to_check:
        click.secho('.', fg="cyan", nl=False)
        if requests.head(url).status_code != 200:
            raise Exception(f"Missing data URL: {url}")

    # Make the data directory.
    build_data(path, config)
    click.secho()

    # Deal with scripts.
    if scripts := config.get('scripts'):
        for script in scripts:
            for p in paths:
                shutil.copyfile(pathlib.Path(KOSU['scripts-source']) / script, p / script)

    # Make the references folder.
    if refs := config.get('references'):
        ref_path = path.joinpath(KOSU['references-target'])
        ref_path.mkdir()
        for fname in refs:
            shutil.copyfile(pathlib.Path(KOSU['references-source']) / fname, ref_path / fname)

    # Make the environment.yaml file.
    env = build_environment(path, config)

    # Make the README.
    _ = build_readme(path, config)

    # Zip it.
    if zip or upload:
        zipped = shutil.make_archive(course, 'zip', root_dir=build_path, base_dir=course)
        click.secho(f"Created {zipped}", fg="green")

    # Upload to AWS.
    if upload:
        success = upload_zip(zipped)
        if success:
            click.secho(f"Uploaded {zipped}", fg="green")
            link  = f"https://{KOSU['s3-bucket']}.s3.amazonaws.com/"
            link += f"{course}.zip"
            click.secho(f"File link: {link}", fg="green")
        if not zip:
            pathlib.Path(f'{course}.zip').unlink()

    # Remove build.
    if clean:
        shutil.rmtree(path)
        click.secho(f"Removed build files.", fg="red")

    return env


def build_notebooks(path, config):
    """
    Process the notebook files. We'll look at three sections of the
    config: curriculum (which contains non-notebook items too),
    extras (which are listed in the README), and demos (which are not).
    """
    # Make the various directories.
    m_path = path.joinpath(KOSU['master-target'])
    m_path.mkdir()
    nb_path = path.joinpath(KOSU['notebooks-target'])
    nb_path.mkdir()
    if config.get('demos'):
        demo_path = path.joinpath(KOSU['demos-target'])
        demo_path.mkdir()
    else:
        demo_path = None

    all_items = [f for items in config['curriculum'].values() for f in items]
    notebooks = list(filter(lambda item: '.ipynb' in item, all_items))
    notebooks += config.get('extras', list())
    images_to_copy = []
    data_urls_to_check = []
    click.secho('Processing notebooks ',fg="cyan" , nl=False)
    for notebook in notebooks:
        infile = pathlib.Path(KOSU['notebooks-source']) / notebook
        outfile = nb_path / notebook
        images, data_urls = process_notebook(infile, outfile)
        images_to_copy.extend(images)
        data_urls_to_check.extend(data_urls)
        shutil.copyfile(infile, m_path / notebook)
        # Clear the outputs in the master file.
        _ = os.system("nbstripout {}".format(m_path / notebook))
        click.secho('+', fg="cyan", nl=False)
    notebooks = config.get('demos', list())
    for notebook in notebooks:
        infile = pathlib.Path(KOSU['notebooks-source']) / notebook
        outfile = demo_path / notebook
        images, data_urls = process_notebook(infile, outfile, demo=True)
        images_to_copy.extend(images)
        data_urls_to_check.extend(data_urls)
        shutil.copyfile(infile, m_path / notebook)
        # Clear the outputs in the master file.
        _ = os.system("nbstripout {}".format(m_path / notebook))
        click.secho('+', fg="cyan", nl=False)
    click.secho()
    if images_to_copy:
        img_path = path.joinpath(KOSU['images-target'])
        img_path.mkdir()
        for image in images_to_copy:
            shutil.copyfile(pathlib.Path(KOSU['images-source']) / image, img_path / image)

    return m_path, nb_path, demo_path, data_urls_to_check


def build_environment(path, config):
    """Construct the environment.yaml file for this course."""
    # Get the base environment.
    with open(f'environment.yaml', 'r') as f:
        try:
            deps = yaml.safe_load(f)
        except yaml.YAMLError as e:
            print(e)

    # Now add the course-specific stuff from the config.
    name = config.get('environment', config['course']).lower()
    conda = {'name': name}
    conda.update(deps)
    if isinstance(conda['dependencies'][-1], dict):
        pip = conda['dependencies'].pop()
    if p := config.get('pip'):
        pip['pip'].extend(p)
    if c := config.get('conda'):
        conda['dependencies'].extend(c)
    conda['dependencies'].append(pip)

    # Write the new environment file to the course directory.
    # Despite YAML recommended practice, we need to use .yml for conda.
    with open(path / 'environment.yml', 'w') as f:
        f.write(yaml.dump(conda, default_flow_style=False, sort_keys=False))
    return conda


def build_readme(path, config):
    """Build the README.md using Jinja2 templates. Note that there
    is a reasonable amount of magic happening at the template level,
    especially text formatting. So if you're looking to change
    something in the README.md, it's probably in there somewhere.
    """
    content = dict(env=config['course'],
                   title=config['title'],
                   curriculum=config.get('curriculum'),
                   extras=config.get('extras'),
                  )
    template = env.get_template('README.md')
    with open(path / 'README.md', 'w') as f:
        f.write(template.render(**content))
    return


def build_data(path, config):
    """Build the data directory. Files must exist in the given path
    of the bucket of AWS S3, per the control file.
    """
    data_path = path.joinpath('data')
    data_path.mkdir()

    s3path = KOSU.get('s3-path', '')
    s3bucket = KOSU.get('s3-bucket')
    data_url = config.get('data_url')
    if data_url is None:
        if s3bucket is None:
            raise TypeError("No data_url or s3-bucket specified.")
        data_url = f"https://{s3bucket}.s3.amazonaws.com/{s3path}{'/' if s3path else ''}"

    if datasets := config.get('data'):
        for fname in datasets:
            click.secho('+',fg="cyan", nl=False)
            fpath = data_path / fname
            if not fpath.exists():
                url = f"{data_url}{fname}"
                urlretrieve(url, fpath)
            if fpath.suffix == '.zip':
                # Inflate and delete the zip.
                with zipfile.ZipFile(fpath, 'r') as z:
                    z.extractall(data_path)
                fpath.unlink()
    else:
        data_path.joinpath('folder_should_be_empty.txt').touch()
    return


def upload_zip(file_name, bucket=None, object_name=None):
    """Upload a file to an S3 bucket

    :param file_name: File to upload
    :param bucket: Bucket to upload to
    :param object_name: S3 object name. If not specified then file_name is used
    :return: True if file was uploaded, else False
    """
    if not AWS_AVAILABLE:
        m = "AWS upload is not available. You need to install boto3 and botocore, "
        m += "and set up AWS credentials. See the documentation for more information."
        raise Exception(m)

    if bucket is None:
        bucket = KOSU['s3-bucket']

    # If S3 object_name was not specified, use file_name.
    if object_name is None:
        object_name = os.path.basename(file_name)

    # Upload the file.
    s3_client = boto3.client('s3')
    try:
        args = {'ACL':'public-read'}
        _ = s3_client.upload_file(file_name, bucket, object_name, ExtraArgs=args)
    except ClientError as e:
        warnings.warn("Upload to S3 failed:", e)
        return False
    return True



from pathlib import Path

from click.testing import CliRunner

from kosu import cli


def test_help():
    """
    Test the help shows up.
    """
    runner = CliRunner()
    result = runner.invoke(cli, ['--help'])
    assert result.exit_code == 0
    assert 'Show this message and exit.' in result.output


def test_missing():
    """
    Test the app does the right thing with a missing arg.
    """
    runner = CliRunner()
    result = runner.invoke(cli, ['test'])
    assert result.exit_code == 2
    assert "Error: Missing argument 'COURSE'" in result.output


def test_clean():
    """
    Test the clean function runs and removes the build folder.
    """
    runner = CliRunner()
    with runner.isolated_filesystem():
        _ = Path('build').mkdir()
        result = runner.invoke(cli, ['clean', '--all'])
        assert result.exit_code == 0
        assert "Finished." in result.output
        assert not Path('build').is_file()


def test_init():
    """
    Test that project initialization works.
    """
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['init', '--yes'])
        # These seem to cause problems, should be enough to check for YAML file.
        # assert result.exit_code == 0
        # assert 'Created example course' in result.output
        assert Path('example_course.yaml').is_file()



from .kosu import *
from .customize import *


from pkg_resources import get_distribution, DistributionNotFound
try:
    VERSION = get_distribution(__name__).version
except DistributionNotFound:
    try:
        from ._version import version as VERSION
    except ImportError:
        raise ImportError(
            "Failed to find (autogenerated) _version.py. "
            "This might be because you are installing from GitHub's tarballs, "
            "use the PyPI ones."
            )
__version__ = VERSION




import sys
import glob
import re


def simplify_credits(html):
    """
    Replace the credit part of the HTML footer. Return the new text.
    """
    s = r'<a class="muted-link" href="https://pradyunsg\.me">@pradyunsg</a>\'s'
    pattern = re.compile(s)
    html = pattern.sub(r'', html)

    s = r'Copyright &#169; 2022, Agile Scientific'
    pattern = re.compile(s)
    new_s = '&#169; 2022, Agile Scientific | <a href="https://creativecommons.org/licenses/by/4.0/">CC BY</a>'
    html = pattern.sub(new_s, html)

    return html


def main(path):
    """
    Process the HTML files in path, save in place (side-effect).
    """
    fnames = glob.glob(path.strip('/') + '/*.html')
    for fname in fnames:
        with open(fname, 'r+') as f:
            html = f.read()

            new_html = simplify_credits(html)

            f.seek(0)
            f.write(new_html)
            f.truncate()
    return


if __name__ == '__main__':
    _ = main(sys.argv[1])
